using Android.Content.Res;
using Java.Util.Zip;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using System;
using System.Collections.Generic;

namespace ExampleScreenEffect
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        SpriteFont font;

        private const string TAG = "Game1";

        private Texture2D _source = null;
        private Texture2D _light = null;

        private Effect _lightingEffect = null;
        private EffectParameter _lightingAlpha = null;

        private Effect LoadEffect(string path)
        {
            byte[] bytes = null;
            try
            {
                AssetManager assetManager = Activity.ApplicationContext.Assets;
                System.IO.Stream inputStream = assetManager.Open(path, Access.Buffer);

                byte[] buffer = new byte[1024];
                ZipInputStream zis = new ZipInputStream(inputStream);
                ZipEntry ze = zis.NextEntry;
                while (ze != null)
                {
                    String filePath = ze.Name;
                    //Log.i(TAG, "Found="+ze.getName());
                    //Log.i(TAG, "Open Output="+newFile.getAbsoluteFile());
                    int totalLength = 0;
                    int len;
                    List<byte[]> chunks = new List<byte[]>();
                    while ((len = zis.Read(buffer)) > 0)
                    {
                        Android.Util.Log.Info(TAG, string.Format("Read: {0} bytes", len));
                        totalLength += len;
                        byte[] chunk = new byte[len];
                        Array.Copy(buffer, chunk, len);
                        chunks.Add(chunk);
                    }
                    bytes = new byte[totalLength];
                    int index = 0;
                    foreach (byte[] chunk in chunks)
                    {
                        Array.Copy(chunk, 0, bytes, index, chunk.Length);
                        index += chunk.Length;
                    }
                    ze = zis.NextEntry;
                }
                zis.Close();
                inputStream.Close();

#if VERBOSE_LOG
                Log.Info(LOG_TAG, string.Format("Successfully loaded the signing key"));
#endif
            }
            catch (Exception ex)
            {
                Android.Util.Log.Error(TAG, string.Format("Exception reading={0}", ex));
            }

            Effect effect = null;
            if (null != bytes)
            {
                try
                {
                    effect = new Effect(GraphicsDevice, bytes);
                }
                catch (Exception ex)
                {
                    Android.Util.Log.Error(TAG, string.Format("Loading Effect Exception={0}", ex));
                }
            }

            return effect;
        }

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);

            Content.RootDirectory = "Content";

            graphics.IsFullScreen = true;
            graphics.PreferredBackBufferWidth = 1920;
            graphics.PreferredBackBufferHeight = 1020;
            graphics.SupportedOrientations = DisplayOrientation.LandscapeLeft | DisplayOrientation.LandscapeRight;
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // TODO: use this.Content to load your game content here
            font = Content.Load<SpriteFont>("spriteFont1");

            _light = Content.Load<Texture2D>("light");
            _source = Content.Load<Texture2D>("source");

            try
            {
                Android.Util.Log.Info(TAG, "...Loading Lighting Effect");
                _lightingEffect = LoadEffect(@"LightingEffectOuya.zip");
                _lightingAlpha = _lightingEffect.Parameters["Alpha"];
                _lightingAlpha.SetValue(0.2f);
            }
            catch (Exception ex)
            {
                Android.Util.Log.Error(TAG, string.Format("Exception loading lighting: {0}", ex));
            }
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
            {
                Exit();
            }

            // TODO: Add your update logic here

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            graphics.GraphicsDevice.Clear(Color.CornflowerBlue);

            spriteBatch.Begin();
            spriteBatch.DrawString(font, "Hello from MonoGame!", new Vector2(16, 16), Color.White);
            spriteBatch.Draw(_source, new Vector2(200, 200), null, Color.White, 0f, Vector2.Zero, 1f, SpriteEffects.None, 0f);
            spriteBatch.Draw(_light, new Vector2(600, 200), null, Color.White, 0f, Vector2.Zero, 1f, SpriteEffects.None, 0f);
            spriteBatch.End();

            // PROBLEM: ***
            //GraphicsDevice.Textures[1] = _light; // some how the _light Texture2D is being disposed when this is set
            //_lightingEffect.Parameters["LightSampler"].SetValue(_light); // this also causes the _light texture2D to be disposed

            spriteBatch.Begin(SpriteSortMode.Texture, BlendState.AlphaBlend, SamplerState.PointClamp, DepthStencilState.None, RasterizerState.CullNone, _lightingEffect, Matrix.Identity);
            spriteBatch.Draw(_source, new Vector2(1000, 600), null, Color.White, 0f, Vector2.Zero, 1f, SpriteEffects.None, 0f);
            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
